/**
 * LICENCIA LGPL:
 * 
 * Esta librera es Software Libre; Usted puede redistribuirla y/o modificarla
 * bajo los trminos de la GNU Lesser General Public License (LGPL) tal y como 
 * ha sido publicada por la Free Software Foundation; o bien la versin 2.1 de 
 * la Licencia, o (a su eleccin) cualquier versin posterior.
 * 
 * Esta librera se distribuye con la esperanza de que sea til, pero SIN 
 * NINGUNA GARANTA; tampoco las implcitas garantas de MERCANTILIDAD o 
 * ADECUACIN A UN PROPSITO PARTICULAR. Consulte la GNU Lesser General Public 
 * License (LGPL) para ms detalles
 * 
 * Usted debe recibir una copia de la GNU Lesser General Public License (LGPL) 
 * junto con esta librera; si no es as, escriba a la Free Software Foundation 
 * Inc. 51 Franklin Street, 5 Piso, Boston, MA 02110-1301, USA o consulte
 * <http://www.gnu.org/licenses/>.
 *
 * Copyright 2011 Agencia de Tecnologa y Certificacin Electrnica
 */
package es.accv.arangi.signature;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;

import org.apache.log4j.Logger;

import es.accv.arangi.base.algorithm.DigitalSignatureAlgorithm;
import es.accv.arangi.base.certificate.Certificate;
import es.accv.arangi.base.document.IDocument;
import es.accv.arangi.base.exception.certificate.NormalizeCertificateException;
import es.accv.arangi.base.exception.device.LoadingObjectException;
import es.accv.arangi.base.exception.document.HashingException;
import es.accv.arangi.base.exception.signature.NoDocumentToSignException;
import es.accv.arangi.base.exception.signature.SignatureException;
import es.accv.arangi.base.util.validation.ValidationResult;
import es.accv.arangi.device.ACCVDeviceManager;
import es.accv.arangi.util.ArangiUtil;

/**
 * Clase que representa una firma en formato CMS.<br><br>
 * 
 * Esta clase simplifica la funcionalidad de su clase padre al utilizar un algoritmo
 * de firma fijo (SHA1WithRSA), no permitir generar firmas attached y validar slo 
 * los certificados tratados por Arang. Es capaz de validar firmas tanto attached 
 * como detached. 
 * 
 * @author <a href="mailto:jgutierrez@accv.es">Jos M Gutirrez</a>
 */
public class CMSSignature extends es.accv.arangi.base.signature.CMSSignature {

	/*
	 * Logger de la clase
	 */
	static Logger logger = Logger.getLogger(CMSSignature.class);
	
	/**
	 * Obtiene la firma de un stream de lectura.
	 * 
	 * @param isSignature Stream de lectura a la firma en formato CMS
	 * @throws IOException Error leyendo el stream de lectura o la firma proporcionada no parece 
	 * 	estar en formato DER
	 * @throws NormalizeCertificateException El certificado de la firma no puede ser normalizado
	 * 	al formato esperado por el proveedor criptogrfico de Arangi
	 * @throws SignatureException Error construyendo la firma
	 */
	public CMSSignature(InputStream isSignature) throws NormalizeCertificateException, SignatureException, IOException {
		super(isSignature);
	}

	
	/**
	 * Obtiene la firma de un fichero.
	 * 
	 * @param fileSignature Fichero con la firma en formato CMS
	 * @throws IOException Error leyendo el fichero o la firma proporcionada no parece estar en formato DER
	 * @throws NormalizeCertificateException El certificado de la firma no puede ser normalizado
	 * 	al formato esperado por el proveedor criptogrfico de Arangi
	 * @throws SignatureException Error construyendo la firma
	 */
	public CMSSignature(File fileSignature) throws NormalizeCertificateException, SignatureException, IOException {
		super(fileSignature);
	}

	/**
	 * Obtiene la firma de un array de bytes.
	 * 
	 * @param signature Firma en formato CMS
	 * @throws NormalizeCertificateException El certificado de la firma no puede ser normalizado
	 * 	al formato esperado por el proveedor criptogrfico de Arangi
	 * @throws SignatureException Error construyendo la firma
	 */
	public CMSSignature(byte[] signature) throws NormalizeCertificateException, SignatureException {
		super(signature);
	}
	
	/**
	 * Construye una firma en formato CMS en base a los bytes de las firmas y
	 * los certificados con los que se realizaron stas.
	 * 
	 * @param signatureBytes Bytes de las firmas
	 * @param certificates Certificados con los que se realiz la firma
	 * @throws SignatureException Error construyendo la firma
	 */
	public CMSSignature(byte[][] signatureBytes, Certificate[] certificates)
			throws SignatureException {
		super(signatureBytes, certificates);
	}
	
	/**
	 * Obtiene un objeto {@link CMSSignature CMSSignature} al firmar el documento con los
	 * dispositivos pasados como parmetro (ms de un dispositivo ocasionar una firma
	 * mltiple).
	 * 
	 * @param managers Dispositivos criptogrficos
	 * @param document Documento a firmar
	 * @throws HashingException No es posible obtener el hash del documento o su versin en 
	 * 	formato DER durante el proceso de firma
	 * @throws LoadingObjectException No ha sido posible cargar la clave privada usada para
	 * 	realizar la firma
	 * @throws SignatureException Error durante el proceso de firma
	 */
	public static CMSSignature sign (ACCVDeviceManager[] managers, IDocument document) throws HashingException, LoadingObjectException, SignatureException {
		return sign(managers, document, false);
	}
	
	/**
	 * Obtiene un objeto {@link CMSSignature CMSSignature} al firmar el documento con los
	 * dispositivos pasados como parmetro (ms de un dispositivo ocasionar una firma
	 * mltiple).
	 * 
	 * @param managers Dispositivos criptogrficos
	 * @param document Documento a firmar
	 * @param isAttached Atached o detached
	 * @throws HashingException No es posible obtener el hash del documento o su versin en 
	 * 	formato DER durante el proceso de firma
	 * @throws LoadingObjectException No ha sido posible cargar la clave privada usada para
	 * 	realizar la firma
	 * @throws SignatureException Error durante el proceso de firma
	 */
	public static CMSSignature sign (ACCVDeviceManager[] managers, IDocument document, boolean isAttached) throws HashingException, LoadingObjectException, SignatureException {
		logger.debug ("[CMSSignature.sign]::Entrada::" + Arrays.asList(new Object[] { managers, document, isAttached } ));
		
		//-- Obtener la lista de bytes de firma y certificados con los que se realizan
		ArrayList alSignatureBytes = new ArrayList ();
		Certificate [] certificates = new Certificate [managers.length];
		String [] algorithms = new String [managers.length];
		for (int i = 0; i < managers.length; i++) {
			//-- Obtener la firma
			alSignatureBytes.add(managers[i].signDocument(document));
			
			//-- Aadir algoritmo (SHA1WithRSA)
			algorithms[i] = DigitalSignatureAlgorithm.SHA1_RSA;
			
			//-- Obtener el certificado
			try {
				certificates[i] = new Certificate (managers[i].getSignatureCertificate());
			} catch (NormalizeCertificateException e) {
				logger.info("[CMSSignature.sign]::El certificado de la firma no ha podido ser normalizado a un formato reconocido " +
						"por el proveedor criptogrfico de Arangi ", e);
				throw new SignatureException ("El certificado de la firma no ha podido ser normalizado a un formato reconocido por el " +
						"proveedor criptogrfico de Arangi ", e);
			}
		}
		byte[][] signaturesBytes = (byte[][])alSignatureBytes.toArray(new byte[0][0]);
		
		//-- Obtener la firma CMS
		byte[] cmsSignature;
		if (isAttached) {
			cmsSignature = createPKCS7CMS (signaturesBytes, certificates, document, algorithms, OID_FORMATO_FIRMA);
		} else {
			cmsSignature = createPKCS7CMS (signaturesBytes, certificates, null, algorithms, OID_FORMATO_FIRMA);
		}
		
		//-- Obtener el objeto
		try {
			return new CMSSignature (cmsSignature);
		} catch (NormalizeCertificateException e) {
			return null;
		}
	}

	/**
	 * Comprueba que las firmas son correctas en firmas attached y sus certificados son vlidos. Slo
	 * sern validados los certificados tratados por Arang.<br><br>
	 * 
	 * Sobre la validacin de certificados hay que tener en cuenta:<br>
	 * <ul>
	 * 	<li>Si existe un sello de tiempos, ste slo ser til mientras el certificado 
	 *  no caduque. Despus, al ser imposible obtener la informacin de revocacin
	 *  para este certificado, este mtodo devolver siempre un resultado falso aunque el 
	 *  certificado fuera vlido cuando se gener la firma .</li>
	 * 	<li>Si la firma incluye informacin de revocacin (CRLs o respuestas OCSP) sta 
	 * 	si que se tendr en cuenta aunque el certificado haya caducado: concepto de
	 * 	firma longeva</li>
	 * </ul><br><br>
	 * 
	 * IMPORTANTE: este mtodo slo puede ser utilizado si la firma es attached (el documento
	 * que origin la firma se incluye en sta). Si no es as utilizar el mtodo con el mismo nombre 
	 * pero con el documento que origin la firma como parmetro.
	 * 
	 * @return Para cada certificado el resultado de comprobar si la firma es correcta y el certificado es
	 * 	vlido
	 * @throws SignatureException Error tratando el objeto firma
	 * @throws HashingException Error obteniendo el hash del documento
	 * @throws NormalizeCertificateException Alguno de los certificados no puede ser 
	 * 	normalizado al formato reconocido por el proveedor criptogrfico de Arang o su 
	 * 	firma no es correcta o no puede ser analizada
	 * @throws NoDocumentToSignException La firma no es attached por lo que no hay documento con
	 * 	el que validarla. Utilizar este mismo mtodo pero pasndole el documento que origin la
	 * 	firma
	 */
	public ValidationResult[] isValid() throws HashingException, SignatureException, NormalizeCertificateException,
			NoDocumentToSignException {
		return super.isValid(ArangiUtil.getACCVCaList());
	}
	
	/**
	 * Comprueba que las firmas son correctas y sus certificados son vlidos. Slo
	 * sern validados los certificados tratados por Arang.<br><br> 
	 * 
	 * Sobre la validacin de certificados hay que tener en cuenta:<br>
	 * <ul>
	 * 	<li>Si existe un sello de tiempos, ste slo ser til mientras el certificado 
	 *  no caduque. Despus, al ser imposible obtener la informacin de revocacin
	 *  para este certificado, este mtodo devolver siempre un resultado falso aunque el 
	 *  certificado fuera vlido cuando se gener la firma .</li>
	 * 	<li>Si la firma incluye informacin de revocacin (CRLs o respuestas OCSP) sta 
	 * 	si que se tendr en cuenta aunque el certificado haya caducado: concepto de
	 * 	firma longeva</li>
	 * </ul>
	 * 
	 * @param document documento firmado en el CMS.
	 * @return Para cada certificado resultado de comprobar si la firma es correcta y el certificado es
	 * 	vlido
	 * @throws SignatureException Error tratando el objeto firma
	 * @throws HashingException Error obteniendo el hash del documento
	 * @throws NormalizeCertificateException Alguno de los certificados no puede ser 
	 * 	normalizado al formato reconocido por el proveedor criptogrfico de Arangi o su 
	 * 	firma no es correcta o no puede ser analizada
	 */
	public ValidationResult[] isValid (IDocument document) throws HashingException, SignatureException, NormalizeCertificateException {
		return super.isValid(document, ArangiUtil.getACCVCaList());
	}

	
}
